"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspaceFoundationCreator = void 0;
const creator_utils_1 = require("@knapsack/creator-utils");
const file_utils_1 = require("@knapsack/file-utils");
const utils_1 = require("@knapsack/utils");
const path_1 = require("path");
const { pkg: { version: ksVersion }, } = (0, file_utils_1.findUpPkgJson)(__dirname);
/**
 * Alter the `knapsack.config.js` file while avoiding Regex or AST as much as possible
 */
function addRendererToConfig({ knapsackConfig, rendererExportName, rendererPkgName, isDefaultExport, rendererConfig = {}, }) {
    return knapsackConfig
        .split('\n')
        .flatMap((line, i) => {
        if (i === 0) {
            // insert `require` statements at top of file
            if (isDefaultExport) {
                return [
                    `const ${rendererExportName} = require('${rendererPkgName}');`,
                    line,
                ];
            }
            return [
                `const { ${rendererExportName} } = require('${rendererPkgName}');`,
                line,
            ];
        }
        const rendererArrayStart = 'templateRenderers: [';
        if (line.includes(rendererArrayStart)) {
            return [
                line.replace(rendererArrayStart, `${rendererArrayStart}new ${rendererExportName}(${rendererConfig ? JSON.stringify(rendererConfig, null, '  ') : ''}),`),
            ];
        }
        return [line];
    })
        .join('\n');
}
async function addRendererToWorkspace({ knapsackConfigPath, rendererExportName, rendererPkgName, rendererConfig, pkgJsonPath, isDefaultExport, }) {
    await Promise.all([
        creator_utils_1.tasks.updateFile({
            filePath: knapsackConfigPath,
            alterFile: (knapsackConfig) => {
                return addRendererToConfig({
                    knapsackConfig,
                    rendererExportName,
                    rendererPkgName,
                    rendererConfig,
                    isDefaultExport,
                });
            },
        }),
        creator_utils_1.tasks.updatePkgJson({
            dir: pkgJsonPath,
            pkg: {
                devDependencies: {
                    [rendererPkgName]: `^${ksVersion}`,
                },
            },
        }),
    ]);
}
const renderersInfo = [
    {
        value: 'react',
        name: 'React',
        rendererExportName: 'KnapsackReactRenderer',
        isDefaultExport: false,
    },
    {
        value: 'vue',
        name: 'Vue',
        rendererExportName: 'KnapsackVueRenderer',
        isDefaultExport: false,
    },
    {
        value: 'html',
        name: 'HTML',
        rendererExportName: 'KnapsackHtmlRenderer',
        isDefaultExport: true,
    },
    {
        value: 'web-components',
        name: 'Web Components',
        rendererExportName: 'KnapsackWebComponentRenderer',
        isDefaultExport: false,
    },
    {
        value: 'twig',
        name: 'Twig',
        rendererExportName: 'KnapsackTwigRenderer',
        isDefaultExport: true,
    },
    {
        value: 'hbs',
        name: 'Handlebars',
        rendererExportName: 'KnapsackHbsRenderer',
        isDefaultExport: false,
    },
];
const packageJson = (ksConfigRelativePath) => {
    return {
        scripts: {
            'ks:build': `knapsack --config ${ksConfigRelativePath} build`,
            'ks:start': `knapsack --config ${ksConfigRelativePath} start`,
            'ks:serve': `NODE_ENV=production knapsack --config ${ksConfigRelativePath} serve`,
            'ks:test': `knapsack --config ${ksConfigRelativePath} test`,
        },
        devDependencies: {
            '@knapsack/app': `^${ksVersion}`,
        },
    };
};
const monorepoPackageJson = (orgName) => {
    return {
        name: `${orgName}/knapsack`,
        version: '1.0.0',
        description: '',
        scripts: {
            build: 'knapsack build',
            start: 'knapsack start',
            serve: 'NODE_ENV=production knapsack serve',
            test: 'knapsack test',
        },
        dependencies: {},
        devDependencies: {
            '@knapsack/app': `^${ksVersion}`,
        },
        author: 'KnapsackBot 53622700+KnapsackBot@users.noreply.github.com',
        license: 'ISC',
    };
};
exports.workspaceFoundationCreator = (0, creator_utils_1.createCreator)({
    id: 'workspace-foundation',
    title: 'Workspace Foundation Creator',
    getQuestions: async () => {
        return {
            siteId: {
                type: 'text',
            },
            renderers: {
                type: 'choices',
                choices: renderersInfo,
            },
            ...(!(0, file_utils_1.isMonorepo)()
                ? {}
                : {
                    packageDir: {
                        type: 'text',
                        title: 'Package directory',
                        default: './packages',
                    },
                    orgName: {
                        type: 'text',
                        title: 'Organization namespace',
                        default: '@my-org',
                    },
                }),
        };
    },
    getTasks: async ({ answers, config: { 
    /** base directory destination for writing files. usually CWD */
    dest, }, }) => {
        const { renderers, packageDir, orgName } = answers;
        if (!answers.siteId) {
            throw new Error(`Missing siteId`);
        }
        if ((0, utils_1.isArray)(renderers) && renderers.length === 0) {
            throw new Error(`Must choose at least one renderer`);
        }
        const siteId = (0, utils_1.isString)(answers.siteId)
            ? (0, utils_1.kebabCase)(answers.siteId.toLowerCase())
            : '';
        const baseDir = (0, file_utils_1.isMonorepo)()
            ? (0, path_1.join)(`${packageDir}`, 'knapsack')
            : 'knapsack';
        const dir = (0, path_1.join)(dest, baseDir);
        await (0, file_utils_1.ensureDir)(dir);
        const pkgPath = (0, file_utils_1.isMonorepo)()
            ? (0, path_1.join)(dir, 'package.json')
            : (0, path_1.join)(dest, 'package.json');
        if (!(0, file_utils_1.existsSync)(pkgPath) && !(0, file_utils_1.isMonorepo)()) {
            throw new Error(`Run in a directory with a package.json file`);
        }
        const dataRel = './data';
        const ksConfigPath = (0, path_1.join)(dir, 'knapsack.config.js');
        const gitRoot = (0, file_utils_1.findGitRoot)(dir);
        const relGitRoot = (0, path_1.relative)(dir, gitRoot);
        const ksConfigRelativePath = (0, path_1.relative)(process.cwd(), ksConfigPath);
        return [
            {
                title: 'Basic repo files',
                task: async (_, task) => {
                    await creator_utils_1.tasks.updatePkgJson({
                        throwErrorIfAbsent: !(0, file_utils_1.isMonorepo)(),
                        dir: pkgPath,
                        pkg: (0, file_utils_1.isMonorepo)()
                            ? monorepoPackageJson(`${orgName}`)
                            : packageJson(ksConfigRelativePath),
                    });
                },
            },
            {
                title: 'Add knapsack.config.js foundation',
                task: async (_, task) => {
                    await creator_utils_1.tasks.addFile({
                        destination: ksConfigPath,
                        handlePriorFile: 'throwError',
                        contents: `
const { configureKnapsack } = require('@knapsack/app');
const { join } = require('path');
const { version } = require('./${(0, path_1.relative)(dir, pkgPath)}');

module.exports = configureKnapsack({
  data: join(__dirname, '${dataRel}'),
  dist: join(__dirname, './dist'),
  public: join(__dirname, './public'),
  version,
  templateRenderers: [],
  plugins: [],
  cloud: {
    siteId: '${siteId}',
    repoRoot: join(__dirname, '${relGitRoot}'),
  },
});
          `,
                    });
                },
            },
            ...(Array.isArray(renderers)
                ? renderers.map((rendererId) => {
                    const { name, rendererExportName, isDefaultExport, value } = renderersInfo.find((r) => r.value === rendererId) || {};
                    const rendererConfig = (() => {
                        switch (rendererId) {
                            case 'react': {
                                const config = {
                                    webpackConfig: {
                                        module: {
                                            rules: [],
                                        },
                                    },
                                };
                                return config;
                            }
                            case 'hbs': {
                                const config = {};
                                return config;
                            }
                            case 'twig': {
                                const config = {
                                    src: {
                                        roots: [],
                                        namespaces: [],
                                    },
                                };
                                // need to do this since it's an `interface` and not a `type` ~ https://stackoverflow.com/a/67219058/1033782
                                return { ...config };
                            }
                            case 'vue': {
                                const config = { webpackConfig: { module: { rules: [] } } };
                                return config;
                            }
                            case 'html':
                            case 'web-components': // has no config
                            default:
                                return null;
                        }
                    })();
                    return {
                        title: `Add ${name} renderer`,
                        task: async () => {
                            await addRendererToWorkspace({
                                pkgJsonPath: pkgPath,
                                knapsackConfigPath: ksConfigPath,
                                rendererExportName,
                                rendererPkgName: `@knapsack/renderer-${value}`,
                                isDefaultExport,
                                rendererConfig,
                            });
                        },
                    };
                })
                : []),
        ];
    },
});
//# sourceMappingURL=workspace-foundation-creator.js.map